from unittest import TestCase, main
from pokdeng.card import Card, Rank, Suit
from pokdeng.exception import DuplicateCards, HandLessThan2Cards, HandMoreThan3Cards
from pokdeng.hand import Hand

class TestHand(TestCase):

    def test_DuplicateCards(self):
        with self.assertRaises(DuplicateCards):
            Hand(cards = [Card(Rank.ACE, Suit.SPADE), Card(Rank.ACE, Suit.SPADE)])

    def test_HandLessThan2Cards(self):
        with self.assertRaises(HandLessThan2Cards):
            Hand(cards = [Card(Rank.ACE, Suit.SPADE)])
    
    def test_HandMoreThan3Cards(self):
        with self.assertRaises(HandMoreThan3Cards):
            Hand(cards = [Card(Rank.NINE, Suit.CLUB), Card(Rank.NINE, Suit.DIAMOND), Card(Rank.NINE, Suit.HEART), Card(Rank.NINE, Suit.SPADE)])

    def test_pok_nine(self):
        hand = Hand(cards = [Card(Rank.ACE, Suit.CLUB), Card(Rank.EIGHT, Suit.HEART)])
        self.assertEqual(hand.pok_nine(), True)

        hand = Hand(cards = [Card(Rank.ACE, Suit.CLUB), Card(Rank.SEVEN, Suit.HEART)])
        self.assertEqual(hand.pok_nine(), False)

        hand = Hand(cards = [Card(Rank.ACE, Suit.CLUB), Card(Rank.TWO, Suit.SPADE), Card(Rank.SIX, Suit.HEART)])
        self.assertEqual(hand.pok_nine(), False)

    def test_pok_eight(self):
        hand = Hand(cards = [Card(Rank.ACE, Suit.CLUB), Card(Rank.SEVEN, Suit.HEART)])
        self.assertEqual(hand.pok_eight(), True)

        hand = Hand(cards = [Card(Rank.ACE, Suit.CLUB), Card(Rank.EIGHT, Suit.HEART)])
        self.assertEqual(hand.pok_eight(), False)

        hand = Hand(cards = [Card(Rank.ACE, Suit.CLUB), Card(Rank.ACE, Suit.SPADE), Card(Rank.SIX, Suit.HEART)])
        self.assertEqual(hand.pok_eight(), False)

    def test_tong(self):
        hand = Hand(cards = [Card(Rank.NINE, Suit.CLUB), Card(Rank.NINE, Suit.DIAMOND), Card(Rank.NINE, Suit.HEART)])
        self.assertEqual(hand.tong(), True)

        hand = Hand(cards = [Card(Rank.NINE, Suit.CLUB), Card(Rank.NINE, Suit.DIAMOND), Card(Rank.TEN, Suit.HEART)])
        self.assertEqual(hand.tong(), False)

        hand = Hand(cards = [Card(Rank.JACK, Suit.CLUB), Card(Rank.JACK, Suit.DIAMOND), Card(Rank.JACK, Suit.HEART)])
        self.assertEqual(hand.tong(), True)
    
    def test_straight_flush(self):
        hand = Hand(cards = [Card(Rank.TWO, Suit.CLUB), Card(Rank.THREE, Suit.CLUB), Card(Rank.FOUR, Suit.CLUB)])
        self.assertEqual(hand.straight_flush(), True)

        hand = Hand(cards = [Card(Rank.JACK, Suit.CLUB), Card(Rank.QUEEN, Suit.CLUB), Card(Rank.KING, Suit.CLUB)])
        self.assertEqual(hand.straight_flush(), True)

        hand = Hand(cards = [Card(Rank.QUEEN, Suit.CLUB), Card(Rank.KING, Suit.CLUB), Card(Rank.ACE, Suit.CLUB)])
        self.assertEqual(hand.straight_flush(), True)

        hand = Hand(cards = [Card(Rank.KING, Suit.CLUB), Card(Rank.ACE, Suit.CLUB), Card(Rank.TWO, Suit.CLUB)])
        self.assertEqual(hand.straight_flush(), False)

        hand = Hand(cards = [Card(Rank.ACE, Suit.CLUB), Card(Rank.TWO, Suit.CLUB), Card(Rank.THREE, Suit.CLUB)])
        self.assertEqual(hand.straight_flush(), False)

        hand = Hand(cards = [Card(Rank.FIVE, Suit.CLUB), Card(Rank.SIX, Suit.CLUB), Card(Rank.SEVEN, Suit.HEART)])
        self.assertEqual(hand.straight_flush(), False)

        hand = Hand(cards = [Card(Rank.FOUR, Suit.CLUB), Card(Rank.FIVE, Suit.CLUB)])
        self.assertEqual(hand.straight_flush(), False)

    def test_straight(self):
        hand = Hand(cards = [Card(Rank.TWO, Suit.CLUB), Card(Rank.THREE, Suit.CLUB), Card(Rank.FOUR, Suit.CLUB)])
        self.assertEqual(hand.straight(), True)

        hand = Hand(cards = [Card(Rank.JACK, Suit.CLUB), Card(Rank.QUEEN, Suit.CLUB), Card(Rank.KING, Suit.CLUB)])
        self.assertEqual(hand.straight(), True)

        hand = Hand(cards = [Card(Rank.QUEEN, Suit.CLUB), Card(Rank.KING, Suit.CLUB), Card(Rank.ACE, Suit.CLUB)])
        self.assertEqual(hand.straight(), True)

        hand = Hand(cards = [Card(Rank.KING, Suit.CLUB), Card(Rank.ACE, Suit.CLUB), Card(Rank.TWO, Suit.CLUB)])
        self.assertEqual(hand.straight(), False)

        hand = Hand(cards = [Card(Rank.ACE, Suit.CLUB), Card(Rank.TWO, Suit.CLUB), Card(Rank.THREE, Suit.CLUB)])
        self.assertEqual(hand.straight(), False)

        hand = Hand(cards = [Card(Rank.FIVE, Suit.CLUB), Card(Rank.SIX, Suit.CLUB), Card(Rank.SEVEN, Suit.HEART)])
        self.assertEqual(hand.straight(), True)

        hand = Hand(cards = [Card(Rank.FOUR, Suit.CLUB), Card(Rank.FIVE, Suit.CLUB)])
        self.assertEqual(hand.straight(), False)

    def test_three_yellow(self):
        hand = Hand(cards = [Card(Rank.JACK, Suit.CLUB), Card(Rank.JACK, Suit.DIAMOND), Card(Rank.QUEEN, Suit.HEART)])
        self.assertEqual(hand.three_yellow(), True)

        hand = Hand(cards = [Card(Rank.JACK, Suit.CLUB), Card(Rank.JACK, Suit.DIAMOND), Card(Rank.JACK, Suit.HEART)])
        self.assertEqual(hand.three_yellow(), True)

        hand = Hand(cards = [Card(Rank.JACK, Suit.CLUB), Card(Rank.JACK, Suit.DIAMOND), Card(Rank.ACE, Suit.HEART)])
        self.assertEqual(hand.three_yellow(), False)
    
    def test_compare_hand(self):
        hand1 = Hand(cards = [Card(Rank.ACE, Suit.CLUB), Card(Rank.EIGHT, Suit.CLUB)])
        hand2 = Hand(cards = [Card(Rank.ACE, Suit.CLUB), Card(Rank.EIGHT, Suit.SPADE)])
        self.assertTrue(hand1 == hand2) # compare deng later

        hand1 = Hand(cards = [Card(Rank.ACE, Suit.CLUB), Card(Rank.EIGHT, Suit.SPADE)])
        hand2 = Hand(cards = [Card(Rank.ACE, Suit.CLUB), Card(Rank.SEVEN, Suit.CLUB)])
        self.assertTrue(hand1 > hand2)

        hand1 = Hand(cards = [Card(Rank.ACE, Suit.CLUB), Card(Rank.EIGHT, Suit.SPADE)])
        hand2 = Hand(cards = [Card(Rank.TWO, Suit.CLUB), Card(Rank.SEVEN, Suit.HEART)])
        self.assertTrue(hand1 == hand2)

        hand1 = Hand(cards = [Card(Rank.ACE, Suit.CLUB), Card(Rank.SEVEN, Suit.SPADE)])
        hand2 = Hand(cards = [Card(Rank.TWO, Suit.CLUB), Card(Rank.THREE, Suit.HEART), Card(Rank.FOUR, Suit.DIAMOND)])
        self.assertTrue(hand1 > hand2)

        hand1 = Hand(cards = [Card(Rank.TWO, Suit.CLUB), Card(Rank.THREE, Suit.HEART), Card(Rank.FOUR, Suit.DIAMOND)])
        hand2 = Hand(cards = [Card(Rank.THREE, Suit.CLUB), Card(Rank.FOUR, Suit.HEART), Card(Rank.FIVE, Suit.DIAMOND)])
        self.assertTrue(hand1 > hand2)

        hand1 = Hand(cards = [Card(Rank.JACK, Suit.CLUB), Card(Rank.KING, Suit.HEART), Card(Rank.QUEEN, Suit.DIAMOND)])
        hand2 = Hand(cards = [Card(Rank.JACK, Suit.SPADE), Card(Rank.KING, Suit.SPADE), Card(Rank.QUEEN, Suit.SPADE)])
        self.assertTrue(hand1 < hand2)

        hand1 = Hand(cards = [Card(Rank.JACK, Suit.SPADE), Card(Rank.KING, Suit.SPADE), Card(Rank.QUEEN, Suit.SPADE)])
        hand2 = Hand(cards = [Card(Rank.TWO, Suit.SPADE), Card(Rank.THREE, Suit.SPADE), Card(Rank.FOUR, Suit.SPADE)])
        self.assertTrue(hand1 < hand2)

        hand1 = Hand(cards = [Card(Rank.JACK, Suit.SPADE), Card(Rank.KING, Suit.SPADE), Card(Rank.QUEEN, Suit.SPADE)])
        hand2 = Hand(cards = [Card(Rank.TWO, Suit.SPADE), Card(Rank.THREE, Suit.SPADE), Card(Rank.FOUR, Suit.HEART)])
        self.assertTrue(hand1 > hand2)

        hand1 = Hand(cards = [Card(Rank.JACK, Suit.SPADE), Card(Rank.KING, Suit.SPADE), Card(Rank.JACK, Suit.HEART)])
        hand2 = Hand(cards = [Card(Rank.ACE, Suit.SPADE), Card(Rank.THREE, Suit.SPADE), Card(Rank.FOUR, Suit.SPADE)])
        self.assertTrue(hand1 > hand2)

        hand1 = Hand(cards = [Card(Rank.ACE, Suit.SPADE), Card(Rank.THREE, Suit.SPADE), Card(Rank.FOUR, Suit.SPADE)])
        hand2 = Hand(cards = [Card(Rank.ACE, Suit.HEART), Card(Rank.THREE, Suit.HEART), Card(Rank.FOUR, Suit.DIAMOND)])
        self.assertTrue(hand1 == hand2) # compare deng later

        hand1 = Hand(cards = [Card(Rank.ACE, Suit.SPADE), Card(Rank.THREE, Suit.SPADE), Card(Rank.FOUR, Suit.CLUB)])
        hand2 = Hand(cards = [Card(Rank.ACE, Suit.HEART), Card(Rank.THREE, Suit.HEART), Card(Rank.FOUR, Suit.DIAMOND)])
        self.assertTrue(hand1 == hand2)

        hand1 = Hand(cards = [Card(Rank.ACE, Suit.SPADE), Card(Rank.THREE, Suit.SPADE)])
        hand2 = Hand(cards = [Card(Rank.ACE, Suit.HEART), Card(Rank.ACE, Suit.SPADE), Card(Rank.TWO, Suit.DIAMOND)])
        self.assertTrue(hand1 == hand2) # compare deng later

        hand1 = Hand(cards = [Card(Rank.ACE, Suit.SPADE), Card(Rank.THREE, Suit.DIAMOND)])
        hand2 = Hand(cards = [Card(Rank.ACE, Suit.HEART), Card(Rank.ACE, Suit.SPADE), Card(Rank.TWO, Suit.DIAMOND)])
        self.assertTrue(hand1 == hand2)

        hand1 = Hand(cards = [Card(Rank.ACE, Suit.SPADE), Card(Rank.TWO, Suit.SPADE)])
        hand2 = Hand(cards = [Card(Rank.ACE, Suit.HEART), Card(Rank.TEN, Suit.HEART), Card(Rank.TWO, Suit.HEART)])
        self.assertTrue(hand1 == hand2) # compare deng later

        hand1 = Hand(cards = [Card(Rank.ACE, Suit.SPADE), Card(Rank.TWO, Suit.HEART)])
        hand2 = Hand(cards = [Card(Rank.ACE, Suit.HEART), Card(Rank.TEN, Suit.HEART), Card(Rank.JACK, Suit.DIAMOND)])
        self.assertTrue(hand1 > hand2)

        hand1 = Hand(cards = [Card(Rank.FIVE, Suit.SPADE), Card(Rank.FOUR, Suit.CLUB)])
        hand2 = Hand(cards = [Card(Rank.NINE, Suit.SPADE), Card(Rank.NINE, Suit.DIAMOND)])
        self.assertTrue(hand1 > hand2)

if __name__ == '__main__':
    main()



from decimal import Decimal
from typing import Dict, List

from pokdeng.card import Deck
from pokdeng.cardholder import Dealer, Player
from pokdeng.exception import GameLessThan1Player, GameMoreThan16Players, GameNoDealer, GamePocketMismatch
from pokdeng.hand import Hand
from pokdeng.idgenerator import CardHolderId
from pokdeng.pocket import Pocket

class Game:
    def __init__(self, dealer: Dealer, players: List[Player], pockets: Dict[CardHolderId, Pocket], verbose: bool = False):
        if dealer is None:
            raise GameNoDealer()
        if (len(players)) < 1:
            raise GameLessThan1Player()
        if len(players) > 16:
            raise GameMoreThan16Players()
        if ({player for player in players if pockets.get(player.card_holder_id) is not None}) == len(players) and pockets.get(dealer.card_holder_id) is not None:
            raise GamePocketMismatch()

        self.dealer: Dealer = dealer
        self.players: List[Player] = players
        self.pockets: Dict[CardHolderId, Pocket] = pockets

        self.round = 0
        self.verbose = verbose

    def play(self, num_rounds: int):
        for _ in range(num_rounds):
            self.round += 1
            deck: Deck = Deck()
            bets: Dict[CardHolderId, Decimal] = {player.card_holder_id : player.place_bet(self.round, self.pockets[player.card_holder_id]) for player in self.players}
            players = [player for player in self.players if bets.get(player.card_holder_id) is not None]
            hands: Dict[CardHolderId, Hand] = {}

            if len(players) == 0:
                if self.verbose:
                    print(f"[round={self.round}] no players bet")
                continue

            for player in players:
                hands[player.card_holder_id] = Hand(cards = [deck.draw(), deck.draw()])
            hands[self.dealer.card_holder_id] = Hand(cards = [deck.draw(), deck.draw()])

            for player in players:
                player_hand = hands.get(player.card_holder_id)
                if player_hand.pok():
                    continue
                pocket = self.pockets.get(player.card_holder_id)
                if player.draw_card(self.round, pocket, player_hand):
                    player_hand.cards.append(deck.draw())

            dealer_pocket = self.pockets.get(self.dealer.card_holder_id)
            dealer_hand = hands.get(self.dealer.card_holder_id)

            if dealer_hand.pok():
                self.__settle_all(players, bets, hands)
                if self.verbose:
                    print(f"[round={self.round}] settled all because dealer pok")
                continue

            if self.dealer.two_fight_three(self.round, dealer_pocket, dealer_hand):
                players_with_3_cards = [player for player in players if hands.get(player.card_holder_id).num_cards() == 3]
                self.__settle_all(players_with_3_cards, bets, hands)
                dealer_hand.cards.append(deck.draw())
                players_with_2_cards = [player for player in players if hands.get(player.card_holder_id).num_cards() == 2]
                self.__settle_all(players_with_2_cards, bets, hands)
                if self.verbose:
                    print(f"[round={self.round}] settled all after two fight three then three fight two")
                continue
            
            if self.dealer.draw_card(self.round, dealer_pocket, dealer_hand):
                dealer_hand.cards.append(deck.draw())
            
            self.__settle_all(players, bets, hands)
            if self.verbose:
                print(f"[round={self.round}] settled all normally")
    
    def __settle_all(self, players: List[Player], bets: Dict[CardHolderId, Decimal], hands: Dict[CardHolderId, Hand]):
        dealer_hand = hands.get(self.dealer.card_holder_id)
        dealer_hand_deng = dealer_hand.deng()
        for player in players:
            player_hand = hands.get(player.card_holder_id)
            player_hand_deng = player_hand.deng()
            if dealer_hand > player_hand:
                self.__pay_dealer(player, bets, dealer_hand_deng)
            elif dealer_hand < player_hand:
                self.__pay_player(player, bets, player_hand_deng)
            else:
                if dealer_hand_deng > player_hand_deng:
                    self.__pay_dealer(player, bets, dealer_hand_deng - player_hand_deng)
                elif dealer_hand_deng < player_hand_deng:
                    self.__pay_player(player, bets, player_hand_deng - dealer_hand_deng)
                else:
                    pass
    
    def __pay_dealer(self, player: Player, bets: Dict[CardHolderId, Decimal], deng: int):
        bet = bets.get(player.card_holder_id)        
        amount: Decimal = bet * deng

        dealer_pocket = self.pockets.get(self.dealer.card_holder_id)
        player_pocket = self.pockets.get(player.card_holder_id)
        self.__settle_amount(from_pocket = player_pocket, to_pocket = dealer_pocket, amount = amount)
    
    def __pay_player(self, player: Player, bets: Dict[CardHolderId, Decimal], deng: int):
        bet = bets.get(player.card_holder_id)
        amount: Decimal = bet * deng

        dealer_pocket = self.pockets.get(self.dealer.card_holder_id)
        player_pocket = self.pockets.get(player.card_holder_id)
        self.__settle_amount(from_pocket = dealer_pocket, to_pocket = player_pocket, amount = amount)

    def __settle_amount(self, from_pocket: Pocket, to_pocket: Pocket, amount: Decimal):
        from_pocket.subtract_amount(amount)
        to_pocket.add_amount(amount)



from dataclasses import dataclass
from functools import total_ordering
from typing import List

from pokdeng.card import Card
from pokdeng.exception import HandLessThan2Cards, HandMoreThan3Cards, DuplicateCards

@dataclass
@total_ordering
class Hand:
    cards: List[Card]

    def __post_init__(self):
        if self.__have_duplicate_cards():
            raise DuplicateCards()
        if self.num_cards() < 2:
            raise HandLessThan2Cards()
        if self.num_cards() > 3:
            raise HandMoreThan3Cards()

    def __eq__(self, other):
        if not isinstance(other, Hand):
            return NotImplemented
        return (self.pok_nine(), self.pok_eight(), self.tong(), self.straight_flush(), self.straight(), self.three_yellow(), self.score()) ==\
            (other.pok_nine(), other.pok_eight(), other.tong(), other.straight_flush(), other.straight(), other.three_yellow(), other.score())

    def __gt__(self, other):
        if not isinstance(other, Hand):
            return NotImplemented
        return (self.pok_nine(), self.pok_eight(), self.tong(), self.straight_flush(), self.straight(), self.three_yellow(), self.score()) >\
            (other.pok_nine(), other.pok_eight(), other.tong(), other.straight_flush(), other.straight(), other.three_yellow(), other.score())

    def num_cards(self) -> int:
        return len(self.cards)

    def __have_duplicate_cards(self) -> bool:
        return len(set([f"{card.rank.description}{card.suit.description}" for card in self.cards])) < self.num_cards()

    def score(self) -> int:
        return sum([card.rank.score for card in self.cards]) % 10

    def __num_unique_ranks(self) -> int:
        return len(set([card.rank for card in self.cards]))
    
    def __num_unique_suits(self) -> int:
        return len(set([card.suit for card in self.cards]))
    
    def __num_faces(self) -> int:
        return len([card for card in self.cards if card.rank.is_face()])
    
    def pok(self) -> bool:
        return self.pok_eight() or self.pok_nine()

    def pok_eight(self) -> bool:
        return self.num_cards() == 2 and self.score() == 8

    def pok_nine(self) -> bool:
        return self.num_cards() == 2 and self.score() == 9
    
    def tong(self) -> bool:
        return self.num_cards() == 3 and self.__num_unique_ranks() == 1 and self.__num_unique_suits() == 3
    
    def straight_flush(self) -> bool:
        if not self.straight():
            return False
        return self.__num_unique_suits() == 1

    def straight(self) -> bool:
        if self.num_cards() != 3:
            return False
        if self.__num_unique_ranks() != 3:
            return False
        combination = "".join([rank.description for rank in sorted([card.rank for card in self.cards], key = lambda rank: rank.code)])
        return combination in {"234", "345", "456", "567", "678", "789", "89T", "9TJ", "TJQ", "JQK", "QKA"}

    def three_yellow(self) -> bool:
        return self.num_cards() == 3 and self.__num_faces() == 3 and not self.straight()

    def deng(self) -> int:
        if self.num_cards() == 2:
            if self.__num_unique_suits() == 1 or self.__num_unique_ranks() == 1:
                return 2
            return 1
        if self.tong() or self.straight_flush():
            return 5
        if self.straight() or self.three_yellow():
            return 3
        if self.__num_unique_suits() == 1:
            return 3
        return 1
    
    



from dataclasses import dataclass
from decimal import Decimal

from pokdeng.idgenerator import CardHolderId

@dataclass
class Pocket:
    card_holder_id: CardHolderId
    total_amount: Decimal

    def add_amount(self, amount: Decimal):
        self.total_amount += amount
    
    def subtract_amount(self, amount: Decimal):
        self.total_amount -= amount



from abc import ABC, abstractmethod
from decimal import Decimal

from pokdeng.hand import Hand
from pokdeng.idgenerator import CardHolderId
from pokdeng.pocket import Pocket

class CardHolder(ABC):
    def __init__(self, card_holder_id: CardHolderId):
        self.card_holder_id: CardHolderId = card_holder_id

    @abstractmethod
    def draw_card(self, round: int, pocket: Pocket, hand: Hand) -> bool:
        pass

class Dealer(CardHolder):
    @abstractmethod
    def two_fight_three(self, round: int, pocket: Pocket, hand: Hand) -> bool:
        pass

class Player(CardHolder):
    @abstractmethod
    def place_bet(self, round: int, pocket: Pocket) -> Decimal:
        pass

class DefaultDealer(Dealer):
    def __init__(self, card_holder_id: CardHolderId = None):
        if card_holder_id is None:
            card_holder_id = CardHolderId()
        super().__init__(card_holder_id)

    def draw_card(self, round: int, pocket: Pocket, hand: Hand) -> bool:
        return hand.score() < 5

    def two_fight_three(self, round: int, pocket: Pocket, hand: Hand) -> bool:
        return hand.score() > 5

class DefaultPlayer(Player):
    def __init__(self, card_holder_id: CardHolderId = None):
        if card_holder_id is None:
            card_holder_id = CardHolderId()
        super().__init__(card_holder_id)

    def draw_card(self, round: int, pocket: Pocket, hand: Hand) -> bool:
        return hand.score() < 5
    
    def place_bet(self, round: int, pocket: Pocket) -> Decimal:
        bet = Decimal("1")
        if pocket.total_amount - bet > Decimal("0"):
            return bet
        return None



class BizException(Exception):
    pass

class DuplicateCards(BizException):
    def __init__(self):
        self.message = "Hand cannot have duplicate cards!"
        super().__init__(self.message)

class HandLessThan2Cards(BizException):
    def __init__(self):
        self.message = "Hand cannot have less than 2 cards!"
        super().__init__(self.message)

class HandMoreThan3Cards(BizException):
    def __init__(self):
        self.message = "Hand cannot have more than 3 cards!"
        super().__init__(self.message)

class GameLessThan1Player(BizException):
    def __init__(self):
        self.message = "Game cannot have less than 1 player!"
        super().__init__(self.message)

class GameMoreThan16Players(BizException):
    def __init__(self):
        self.message = "Game cannot have more than 16 players!"
        super().__init__(self.message)

class GameNoDealer(BizException):
    def __init__(self):
        self.message = "Game cannot have no dealer!"
        super().__init__(self.message)

class GamePocketMismatch(BizException):
    def __init__(self):
        self.message = "Game cannot have pocket mismatch!"
        super().__init__(self.message)



from dataclasses import dataclass
from enum import Enum
from random import shuffle
from typing import List

class Suit(Enum):
    CLUB = (1, "C")
    DIAMOND = (2, "D")
    HEART = (3, "H")
    SPADE = (4, "S")

    def __init__(self, code: int, description: str):
        self.code = code
        self.description = description

class Rank(Enum):
    TWO = (2, 2, "2")
    THREE = (3, 3, "3")
    FOUR = (4, 4, "4")
    FIVE = (5, 5, "5")
    SIX = (6, 6, "6")
    SEVEN = (7, 7, "7")
    EIGHT = (8, 8, "8")
    NINE = (9, 9, "9")
    TEN = (10, 10, "T")
    JACK = (11, 10, "J")
    QUEEN = (12, 10, "Q")
    KING = (13, 10, "K")
    ACE = (14, 1, "A")

    def __init__(self, code: int, score: int, description: str):
        self.code = code
        self.score = score
        self.description = description

    def is_face(self) -> bool:
        return self in {self.JACK, self.QUEEN, self.KING}

@dataclass
class Card:
    rank: Rank
    suit: Suit

class Deck:
    def __init__(self):
        self.cards: List[Card] = [Card(rank, suit) for rank in Rank for suit in Suit]
        shuffle(self.cards)
    
    def draw(self) -> Card:
        if len(self.cards) > 0:
            return self.cards.pop()
        return None



import setuptools

with open("README.md", "r") as fh:
    long_description = fh.read()

setuptools.setup(
    name="pokdeng",
    version="0.0.0",
    author="Papan Yongmalwong",
    author_email="papillonbee@gmail.com",
    description="pokdeng is a package for simulating rounds of pokdeng games!",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/papillonbee/pokdeng",
    packages=setuptools.find_packages(),
    test_suite='tests',
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
    python_requires='>=3.7',
    install_requires=[
    ]
)
